@@ -83,7 +83,7 @@ class TextChannel(tp.ChannelTypeText):
 
 	@property
 	def _contactKey(self):
-		contactKey = self.__otherHandle.contactID, self.__otherHandle.phoneNumber
+		contactKey = self.__otherHandle.phoneNumber
 		return contactKey
 
 	@gtk_toolbox.log_exception(_moduleLogger)
