@@ -1,12 +1,13 @@
 
 """
 @todo Add params for different state machines update times
-@todo Add option to use screen name as callback
 @todo Get a callback for missed calls to force an update of the voicemail state machine
 @todo Get a callback on an incoming call and if its from GV, auto-pickup
+@todo Decide if we should do what the spec suggests and hold onto a singleton contactlist
 """
 
 
+import os
 import weakref
 import logging
 
@@ -21,6 +22,7 @@ except (ImportError, OSError):
 import constants
 import tp
 import util.coroutines as coroutines
+import util.misc as util_misc
 import gtk_toolbox
 
 import gvoice
@@ -54,12 +56,13 @@ class TheOneRingConnection(
 	_mandatory_parameters = {
 		'account' : 's',
 		'password' : 's',
-		'forward' : 's',
 	}
 	# Overriding a base class variable
 	_optional_parameters = {
+		'forward' : 's',
 	}
 	_parameter_defaults = {
+		'forward' : '',
 	}
 
 	@gtk_toolbox.log_exception(_moduleLogger)
@@ -68,11 +71,12 @@ class TheOneRingConnection(
 		account = unicode(parameters['account'])
 		encodedAccount = parameters['account'].encode('utf-8')
 		encodedPassword = parameters['password'].encode('utf-8')
-		encodedCallback = parameters['forward'].encode('utf-8')
-		if not encodedCallback:
-			raise telepathy.errors.InvalidArgument("User must specify what number GV forwards calls to")
+		encodedCallback = util_misc.normalize_number(parameters['forward'].encode('utf-8'))
+		if encodedCallback and not util_misc.is_valid_number(encodedCallback):
+			raise telepathy.errors.InvalidArgument("Invalid forwarding number")
 
 		# Connection init must come first
+		self.__session = gvoice.session.Session(None)
 		tp.Connection.__init__(
 			self,
 			constants._telepathy_protocol_name_,
@@ -91,16 +95,21 @@ class TheOneRingConnection(
 			encodedAccount,
 			encodedPassword,
 		)
-		self.__callbackNumber = encodedCallback
+		self.__callbackNumberParameter = encodedCallback
 		self.__channelManager = channel_manager.ChannelManager(self)
 
-		self.__session = gvoice.session.Session(None)
 		if conic is not None:
 			self.__connection = conic.Connection()
 			self.__connectionEventId = None
 		else:
 			self.__connection = None
 			self.__connectionEventId = None
+		self.__cachePath = os.sep.join((constants._data_path_, "cache", self.username))
+		try:
+			os.makedirs(self.__cachePath)
+		except OSError, e:
+			if e.errno != 17:
+				raise
 
 		self.set_self_handle(handle.create_handle(self, 'connection'))
 
@@ -120,20 +129,17 @@ class TheOneRingConnection(
 		return self.__credentials[0]
 
 	@property
-	def userAliasType(self):
-		return self.USER_ALIAS_ACCOUNT
+	def callbackNumberParameter(self):
+		return self.__callbackNumberParameter
 
 	def get_handle_by_name(self, handleType, handleName):
 		requestedHandleName = handleName.encode('utf-8')
 		if handleType == telepathy.HANDLE_TYPE_CONTACT:
-			_moduleLogger.info("RequestHandles Contact: %s" % requestedHandleName)
-			requestedContactId, requestedContactNumber = handle.ContactHandle.from_handle_name(
-				requestedHandleName
-			)
-			h = handle.create_handle(self, 'contact', requestedContactId, requestedContactNumber)
+			_moduleLogger.info("get_handle_by_name Contact: %s" % requestedHandleName)
+			h = handle.create_handle(self, 'contact', requestedHandleName)
 		elif handleType == telepathy.HANDLE_TYPE_LIST:
 			# Support only server side (immutable) lists
-			_moduleLogger.info("RequestHandles List: %s" % requestedHandleName)
+			_moduleLogger.info("get_handle_by_name List: %s" % requestedHandleName)
 			h = handle.create_handle(self, 'list', requestedHandleName)
 		else:
 			raise telepathy.errors.NotAvailable('Handle type unsupported %d' % handleType)
@@ -154,8 +160,7 @@ class TheOneRingConnection(
 			telepathy.CONNECTION_STATUS_REASON_REQUESTED
 		)
 		try:
-			cookieFilePath = None
-			self.__session = gvoice.session.Session(cookieFilePath)
+			self.__session.load(self.__cachePath)
 
 			self.__callback = coroutines.func_sink(
 				coroutines.expand_positional(
@@ -169,7 +174,12 @@ class TheOneRingConnection(
 				self.__callback
 			)
 			self.session.login(*self.__credentials)
-			self.session.backend.set_callback_number(self.__callbackNumber)
+			if not self.__callbackNumberParameter:
+				callback = gvoice.backend.get_sane_callback(
+					self.session.backend
+				)
+				self.__callbackNumberParameter = util_misc.normalize_number(callback)
+			self.session.backend.set_callback_number(self.__callbackNumberParameter)
 		except gvoice.backend.NetworkError, e:
 			_moduleLogger.exception("Connection Failed")
 			self.StatusChanged(
@@ -255,6 +265,7 @@ class TheOneRingConnection(
 		self.__callback = None
 
 		self.__channelManager.close()
+		self.session.save(self.__cachePath)
 		self.session.logout()
 		self.session.close()
 		self.__session = None
@@ -268,16 +279,18 @@ class TheOneRingConnection(
 	@gtk_toolbox.log_exception(_moduleLogger)
 	def _on_conversations_updated(self, conv, conversationIds):
 		_moduleLogger.debug("Incoming messages from: %r" % (conversationIds, ))
-		for contactId, phoneNumber in conversationIds:
-			h = handle.create_handle(self, 'contact', contactId, phoneNumber)
+		for phoneNumber in conversationIds:
+			h = self.get_handle_by_name(telepathy.HANDLE_TYPE_CONTACT, phoneNumber)
 			# Just let the TextChannel decide whether it should be reported to the user or not
 			props = self._generate_props(telepathy.CHANNEL_TYPE_TEXT, h, False)
-			channel = self.__channelManager.channel_for_props(props, signal=True)
+			chan = self.__channelManager.channel_for_props(props, signal=True)
 
 	@gtk_toolbox.log_exception(_moduleLogger)
 	def _on_connection_change(self, connection, event):
 		"""
 		@note Maemo specific
+
+		@todo Make this delayed to handle background switching of networks
 		"""
 		status = event.get_status()
 		error = event.get_error()
